from django import template
from django.core.urlresolvers import resolve
from django.core.cache import cache
from django.utils import translation
from django.conf import settings
import logging
from ..import get_class_for_view

from ..settings import (
    SEO_CACHE_PREFIX,
    SEO_CACHE_TIMEOUT,
    SEO_USE_CACHE,
)

from ..fields import (
    TitleTagField,
    MetaTagField,
    KeywordsTagField,
    URLMetaTagField,
    ImageMetaTagField
)


log = logging.getLogger(__name__)

register = template.Library()


class MetadataNode(template.Node):
    """
    Template Tag node for Metadata
    * gets the view name from request
    * obtains metadata model and object
    * print each supported field
    """

    @staticmethod
    def _build_prefix(context, view_name):
        lang = translation.get_language()
        return SEO_CACHE_PREFIX + ':' + view_name + ':' + lang + ':' + context['request'].path

    @staticmethod
    def _check_field_i18n(field):
        """
        Avoid fields that has _XX lang prefix
        """
        if not getattr(settings, 'USE_I18N', None) or field is None:
            return False
        for lang in settings.LANGUAGES:
            if '_'+lang[0] in field.name:
                return True

        return False

    def render(self, context):
        view_name = resolve(context['request'].path).url_name  # resolve view name

        # Check if metadata is in cache
        if SEO_USE_CACHE:
            metadata_html = cache.get(self._build_prefix(context, view_name))
            if metadata_html:
                log.debug("Cache metadata hit for view %s" % view_name)
                return metadata_html

        seo_model = get_class_for_view(view_name)
        try:
            metadata = seo_model.objects.get(view_name=view_name)
            metadata_html = ""
            for field in metadata._meta.fields:
                if not self._check_field_i18n(field) and isinstance(
                        field,
                        (TitleTagField, MetaTagField, KeywordsTagField, URLMetaTagField, ImageMetaTagField)):
                    printed_tag = field.to_python(getattr(metadata, field.name)).print_tag()
                    if printed_tag and printed_tag != "":
                        metadata_html += printed_tag + "\n"
                else:
                    pass
            if metadata_html != "" and SEO_USE_CACHE:
                cache.set(self._build_prefix(context, view_name), metadata_html, SEO_CACHE_TIMEOUT)

            if metadata_html:
                return metadata_html
            else:
                return ""
        except seo_model.DoesNotExist as exc:
            # Skipping error to avoid breaking the view
            log.debug("No metadata found for view %s" % view_name)
            return ""


@register.tag
def view_metadata(context, parser):
    return MetadataNode()




from __future__ import print_function
from django.contrib.admin import ModelAdmin
from django import forms

from .import load_view_names


class BaseMetadataForm(forms.ModelForm):
    view_name = forms.ChoiceField(choices=load_view_names())


class BaseMetadataAdmin(ModelAdmin):
    """
    Overrides default admin to add autodiscovered views into a choice field
    """
    list_display = ['view_name', 'title']
    exclude = []
    form = BaseMetadataForm

    def formfield_for_dbfield(self, db_field, **kwargs):
        formfield = super(BaseMetadataAdmin, self).formfield_for_dbfield(db_field, **kwargs)
        if db_field.name == 'title':
            formfield.widget = forms.Textarea(attrs=formfield.widget.attrs)
        if db_field.name == 'description':
            formfield.widget = forms.Textarea(attrs=formfield.widget.attrs)
        if db_field.name == 'keywords':
            formfield.widget = forms.Textarea(attrs=formfield.widget.attrs)
        if db_field.name == 'og:title':
            formfield.widget = forms.Textarea(attrs=formfield.widget.attrs)
        if db_field.name == 'og:description':
            formfield.widget = forms.Textarea(attrs=formfield.widget.attrs)
        if db_field.name == 'twitter:title':
            formfield.widget = forms.Textarea(attrs=formfield.widget.attrs)
        if db_field.name == 'twitter:description':
            formfield.widget = forms.Textarea(attrs=formfield.widget.attrs)
        return formfield


from __future__ import print_function
from django.db.models.loading import get_model
from django.utils.six import iteritems
from django.core.exceptions import ImproperlyConfigured
from django.core.urlresolvers import RegexURLResolver, RegexURLPattern
import logging
from django.conf import settings as django_settings

log = logging.getLogger(__name__)

from .settings import SEO_MODEL_REGISTRY

#  Global registries
_simple_seo_registry = []
_view_names_registry = []


def _register(model_name, views_related):
    """
    Registers model for the given views. If no views given then all views are bound.
    If more than one seo model is registered, order matter (in case view name is defined in more than one model,
    first registered is applied only
    :param model_name: Model to register
    :param views_related: list of str
    :return:
    """
    if views_related and isinstance(views_related, str):
        if views_related != 'ALL':
            raise ImproperlyConfigured(
                "You must define a list/tuple of view names or 'ALL' in SEO_MODEL_REGISTRY. http://bit.ly/1pmE8vU")
    elif not views_related or not isinstance(views_related, (list, tuple)):
        raise ImproperlyConfigured(
            "You must define a list/tuple of view names or 'ALL' in SEO_MODEL_REGISTRY. http://bit.ly/1pmE8vU")
    _simple_seo_registry.append(
        {
            model_name: views_related
        }
    )


def _get_class_from_name(model_name):
    """
    Returns the Model
    :param model_name: 'app_label.ModelName'
    :return:
    """
    _app_label, _model_name = registry[0].split('.')
    return get_model(_app_label, _model_name)


def get_class_for_view(view):
    """
    Given a view name, give a valid seo model. It's gonna be the first model who contains the view name or has
     all-views scope
    :param view:
    :return: model
    """
    if not view:
        raise ValueError("View must have value")

    global _simple_seo_registry
    for registry_dict in _simple_seo_registry:
        for key, value in iteritems(registry_dict):
            if isinstance(value, (list, tuple)):
                if view in value:
                    return _get_class_from_name(key)
            elif isinstance(value, str) and value == 'ALL':
                return _get_class_from_name(key)
    raise ValueError("No model has been found for view with name %s. Have you registered one?" % view)


try:
    for registry in SEO_MODEL_REGISTRY:
        _views_related = registry[1]
        _register(registry[0], _views_related)

except (IndexError, TypeError, ImportError):
    raise ImproperlyConfigured(
        "SEO_MODEL_REGISTRY setting has bad format. Look at the documentation here http://bit.ly/1pmE8vU"
    )


def _load_pattern(views, pattern, namespace=None):
    """
    Computes RegexURLPattern
    :param pattern: RegexURLPattern
    :param namespace: str
    :return: tuple
    """
    name = getattr(pattern, 'name', None)
    if name:
        if namespace:
            views.append((namespace + ':' + name, namespace + ':' + name))
        else:
            views.append((name, name))


def _load_patterns(views, patterns, namespace=None):
    """
    Computes RegexURLResolver
    :param views: list
    :param patterns: RegexURLResolver
    :param namespace: str
    :return:
    """
    for pattern in patterns:
        if isinstance(pattern, RegexURLPattern):
            _load_pattern(views, pattern, namespace)
        elif isinstance(pattern, RegexURLResolver):
            if namespace and hasattr(pattern, 'namespace') and getattr(pattern, 'namespace'):
                namespace += ':' + pattern.namespace
            elif hasattr(pattern, 'namespace') and getattr(pattern, 'namespace'):
                namespace = pattern.namespace
            _load_patterns(views, pattern.url_patterns, namespace)
        else:
            pass


def load_view_names(urlconf=None):
    """
    Loads view names
    Warning: Only named views are loaded
    :return: generator
    """
    global _view_names_registry
    if len(_view_names_registry):
        return _view_names_registry

    if not urlconf:
        try:
            urlconf = __import__(django_settings.ROOT_URLCONF, {}, {}, [''])
        except Exception as ie:
            raise ImproperlyConfigured("Error occurred while trying to load %s: %s"
                                       % (getattr(settings, 'ROOT_URLCONF', '\'NO settings.ROOT_URLCONF found\''), str(ie)))

    views = []
    for p in urlconf.urlpatterns:
        if isinstance(p, RegexURLPattern):
            _load_pattern(views, p)
        elif isinstance(p, RegexURLResolver):
            _load_patterns(views, p.url_patterns, getattr(p, 'namespace', None))
        else:
            pass
    _view_names_registry.extend(views)
    return _view_names_registry







from django.test import TestCase
from selenium.webdriver.firefox.webdriver import WebDriver as FirefoxDriver
from django.test.testcases import LiveServerTestCase
from simple_seo.models import TestMetadata

from simple_seo.tags import TitleTag, MetaTag, KeywordsTag, BaseTag


class TagPrintingTest(TestCase):
    """
    Testing Tag objects printing functions
    """
    base_tag_selfclosed = None
    base_tag_notselfclosed = None

    def setUp(self):
        self.base_tag_notselfclosed = BaseTag(tag_name='test', self_closed=False, tag_value='Test Tag',
                                              meta_name='meta name',
                                              meta_content='meta content')
        self.base_tag_selfclosed = BaseTag(tag_name='test', self_closed=True, tag_value='Test Tag',
                                           meta_name='meta name',
                                           meta_content='meta content')

    def tearDown(self):
        self.base_tag_selfclosed = None
        self.base_tag_notselfclosed = None

    def test_notselfclosed_tag(self):
        self.assertEqual(self.base_tag_notselfclosed.print_tag(), '<test>Test Tag</test>')

    def test_selfclosed_tag(self):
        self.assertEqual(self.base_tag_selfclosed.print_tag(), '<test name="meta name" content="meta content" />')


class FieldPrintingTest(LiveServerTestCase):
    """
    Testing TagFields objects printing functions
    """

    def setUp(self):
        self.mymetadata = TestMetadata()
        self.mymetadata.view_name = 'template_test'
        self.mymetadata.title = TitleTag(tag_value='Title Test')
        self.mymetadata.author = MetaTag(meta_name='author', meta_content='Test Author')
        self.mymetadata.description = MetaTag(meta_name='description', meta_content='Test Description')
        self.mymetadata.keywords = KeywordsTag(meta_name='keywords', meta_content='test, author')
        # Attr name and class attribute name are not equals, let's do it by setattr
        setattr(self.mymetadata, 'og:title', MetaTag(meta_name='og:title'))
        setattr(self.mymetadata, 'twitter:title', MetaTag(meta_name='twitter:title'))
        setattr(self.mymetadata, 'og:description', MetaTag(meta_name='og:description'))
        setattr(self.mymetadata, 'twitter:description', MetaTag(meta_name='twitter:description'))
        setattr(self.mymetadata, 'og:url', MetaTag(meta_name='og:url', meta_content='http://infantium.com'))
        setattr(self.mymetadata, 'twitter:url', MetaTag(meta_name='twitter:url'))
        setattr(self.mymetadata, 'og:type', MetaTag(meta_name='og:type', meta_content='type'))
        setattr(self.mymetadata, 'twitter:card', MetaTag(meta_name='twitter:card', meta_content='card'))
        setattr(self.mymetadata, 'og:image', MetaTag(meta_name='og:image', meta_content='//infantiumweblog.s3.amazonaws.com/img/partners/animation2/base.png'))
        setattr(self.mymetadata, 'twitter:image', MetaTag(meta_name='twitter:image'))

        self.mymetadata.save()

        self.firefox = FirefoxDriver()

    def tearDown(self):
        self.firefox.quit()

    def test_title_rendering(self):
        self.firefox.get('%s%s' % (self.live_server_url, '/test/'))
        title_element = self.firefox.title
        self.assertIsNotNone(title_element)
        self.assertEqual(title_element, 'Title Test')

    def test_metatags_rendering(self):
        self.firefox.get('%s%s' % (self.live_server_url, '/test/'))

        author_element = self.firefox.find_element_by_xpath('/html/head/meta[@name=\'author\']')
        self.assertIsNotNone(author_element)
        self.assertEqual(author_element.get_attribute('content'), 'Test Author')

        description_element = self.firefox.find_element_by_xpath('/html/head/meta[@name=\'description\']')
        self.assertIsNotNone(description_element)
        self.assertEqual(description_element.get_attribute('content'), 'Test Description')

        keywords_element = self.firefox.find_element_by_xpath('/html/head/meta[@name=\'keywords\']')
        self.assertIsNotNone(keywords_element)
        self.assertEqual(keywords_element.get_attribute('content'), 'test, author')

        og_title_element = self.firefox.find_element_by_xpath('/html/head/meta[@name=\'og:title\']')
        self.assertIsNotNone(og_title_element)
        self.assertEqual(og_title_element.get_attribute('content'), self.firefox.title)  # Should be equal to titleTag
        twitter_title_element = self.firefox.find_element_by_xpath('/html/head/meta[@name=\'twitter:title\']')
        self.assertIsNotNone(twitter_title_element)
        self.assertEqual(twitter_title_element.get_attribute('content'), og_title_element.get_attribute('content'))

        og_description_element = self.firefox.find_element_by_xpath('/html/head/meta[@name=\'og:description\']')
        self.assertIsNotNone(og_description_element)
        self.assertEqual(og_description_element.get_attribute('content'), description_element.get_attribute('content'))
        twitter_description_element = self.firefox.find_element_by_xpath('/html/head/meta[@name=\'twitter:description\']')
        self.assertIsNotNone(twitter_description_element)
        self.assertEqual(twitter_description_element.get_attribute('content'), og_description_element.get_attribute('content'))

        og_url_element = self.firefox.find_element_by_xpath('/html/head/meta[@name=\'og:url\']')
        self.assertIsNotNone(og_url_element)
        self.assertEqual(og_url_element.get_attribute('content'), 'http://infantium.com')
        twitter_url_element = self.firefox.find_element_by_xpath('/html/head/meta[@name=\'twitter:url\']')
        self.assertIsNotNone(twitter_url_element)
        self.assertEqual(twitter_url_element.get_attribute('content'), og_url_element.get_attribute('content'))

        og_image_element = self.firefox.find_element_by_xpath('/html/head/meta[@name=\'og:image\']')
        self.assertIsNotNone(og_image_element)
        self.assertEqual(og_image_element.get_attribute('content'), '//infantiumweblog.s3.amazonaws.com/img/partners/animation2/base.png')
        twitter_image_element = self.firefox.find_element_by_xpath('/html/head/meta[@name=\'twitter:image\']')
        self.assertIsNotNone(twitter_image_element)
        self.assertEqual(twitter_image_element.get_attribute('content'), og_image_element.get_attribute('content'))


from django.conf import settings
from django.core.exceptions import ImproperlyConfigured

SEO_MODEL_REGISTRY = getattr(settings, 'SEO_MODEL_REGISTRY', None)
if SEO_MODEL_REGISTRY is None:
    raise ImproperlyConfigured(
        "You must define SEO_MODEL_REGISTRY setting. Look at the documentation here http://bit.ly/1pmE8vU"
    )

SEO_CACHE_PREFIX = getattr(settings, 'SEO_CACHE_PREFIX', 'simple_seo:')
SEO_CACHE_TIMEOUT = getattr(settings, 'SEO_CACHE_TIMEOUT', 60 * 60 * 24)
SEO_USE_CACHE = getattr(settings, 'SEO_USE_CACHE', False)


from django.db import models

from simple_seo.fields import (
    TitleTagField,
    KeywordsTagField,
    MetaTagField,
    ImageMetaTagField,
    URLMetaTagField,
    BaseTagField)


def _post_init_field_populate(sender, instance, *args, **kwargs):
    """
    Must populate the fields with 'populate_from' value
    Only populates if destination field has empty values
    Saves values, so every populating are only save to database once (first time)
    """
    for field in instance._meta.fields:
        _populate_from_name = getattr(field, 'populate_from', None)
        if _populate_from_name and (isinstance(
                field,
                (BaseTagField, URLMetaTagField, ImageMetaTagField)) or issubclass(
                field.__class__,
                (BaseTagField, URLMetaTagField, ImageMetaTagField))):
            _populate_from_tag = getattr(instance, _populate_from_name, None)
            if _populate_from_tag:
                # Populates meta_content and tag_value values to the destination field
                # Case1: we populate always tag_value if not self_closed in both fields
                # Case2: both are metas self_closed, then copy meta_content
                # Case3: destination is a meta self_closed but origin field is not self_closed: copy tag_value
                destination_tag = getattr(instance, field.name, None)
                if destination_tag is not None:
                    if destination_tag.meta_content is None or destination_tag.meta_content == '':
                        if destination_tag.self_closed and _populate_from_tag.self_closed:
                            destination_tag.meta_content = _populate_from_tag.meta_content
                            setattr(instance, field.name, destination_tag)
                            instance.save()
                        if destination_tag.self_closed and not _populate_from_tag.self_closed:
                            destination_tag.meta_content = _populate_from_tag.tag_value
                            setattr(instance, field.name, destination_tag)
                            instance.save()

                    if destination_tag.tag_value is None or destination_tag.tag_value == '':
                        if not destination_tag.self_closed and not _populate_from_tag.self_closed:
                            destination_tag.tag_value = _populate_from_tag.tag_value
                            setattr(instance, field.name, destination_tag)
                            instance.save()


class BaseMetadata(models.Model):
    """
    Abstract Base Metadata Class
    """
    view_name = models.CharField(max_length=250, null=False, blank=False, choices=(), unique=True, db_index=True)
    title = TitleTagField(null=False, blank=False)
    keywords = KeywordsTagField(null=False, blank=False)
    description = MetaTagField(null=False, blank=False)
    author = MetaTagField(null=False, blank=False)

    def __init__(self, *args, **kwargs):
        super(BaseMetadata, self).__init__(*args, **kwargs)
        models.signals.post_init.connect(_post_init_field_populate, sender=self.__class__)

    class Meta:
        abstract = False


class OpenGraphMetadata(BaseMetadata):
    """
    Abstract Base Metadata Class with Open Graph Tags
    """
    og_title = MetaTagField(name='og:title', max_length=95, null=True, blank=True, populate_from='title')
    og_type = MetaTagField(name='og:type', max_length=15, null=True, blank=True)
    og_image = ImageMetaTagField(name='og:image', upload_to='seo/images/', null=True, blank=True)
    og_url = URLMetaTagField(name='og:url', null=True, blank=True)
    og_description = MetaTagField(name='og:description', max_length=297, null=True, blank=True,
                                  populate_from='description')
    og_admins = MetaTagField(name='og:admins', max_length=297, null=True, blank=True)

    class Meta:
        abstract = False


class TwitterMetadata(BaseMetadata):
    """
    Abstract Base Metadata Class with Twitter Tags
    """
    twitter_title = MetaTagField(name='twitter:title', max_length=70, null=True, blank=True, populate_from='og:title')
    twitter_card = MetaTagField(name='twitter:card', max_length=15, null=True, blank=True)
    twitter_image = ImageMetaTagField(name='twitter:image', upload_to='seo/images/', null=True, blank=True,
                                      populate_from='og:image')
    twitter_description = MetaTagField(name='twitter:description', max_length=200, null=True, blank=True,
                                       populate_from='og:description')

    class Meta:
        abstract = False


class AllMetadata(OpenGraphMetadata):
    """
    Abstract Base Metadata Class with All Tags
    """
    twitter_title = MetaTagField(name='twitter:title', max_length=70, null=True, blank=True, populate_from='og:title')
    twitter_card = MetaTagField(name='twitter:card', max_length=15, null=True, blank=True)
    twitter_image = ImageMetaTagField(name='twitter:image', upload_to='seo/images/', null=True, blank=True,
                                      populate_from='og:image')
    twitter_url = URLMetaTagField(name='twitter:url', null=True, blank=True, populate_from='og:url')
    twitter_description = MetaTagField(name='twitter:description', max_length=200, null=True, blank=True,
                                       populate_from='og:description')

    class Meta:
        abstract = False


try:
    # In case South is installed
    from south.modelsinspector import add_introspection_rules

    add_introspection_rules(
        [],
          [
              "^simple_seo\.fields\.TitleTagField",
              "^simple_seo\.fields\.MetaTagField",
              "^simple_seo\.fields\.URLMetaTagField",
              "^simple_seo\.fields\.ImageMetaTagField",
              "^simple_seo\.fields\.KeywordsTagField",
          ])
except ImportError:
    pass


# Just for testing purposes...
class TestMetadata(AllMetadata):
    """
    My Seo Model
    """
    pass